# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Initialization of the frontend layout across all the steps."""


import webbrowser

from ansys.saf.glow.client.dashclient import DashClient, callback
from ansys_web_components_dash import AwcDashTree
from dash.exceptions import PreventUpdate
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, callback_context, dcc, html
from dash_iconify import DashIconify
import dash_mantine_components as dmc

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{ cookiecutter.__solution_definition_name }}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.pages import about_page, first_page, second_page

step_list = [
    {
        "id": "about_page",
        "text": "About",
        "prefixIcon": {"src": "https://s2.svgbox.net/hero-solid.svg?ic=home"},
        "expanded": True,
    },
    {
        "id": "first_page",
        "text": "First Step",
        "prefixIcon": {"src": "https://s2.svgbox.net/materialui.svg?ic=label"},
        "expanded": True,
    },
    {
        "id": "second_page",
        "text": "Second Step",
        "prefixIcon": {"src": "https://s2.svgbox.net/materialui.svg?ic=label_outline"},
        "expanded": True,
    },
]


layout = html.Div(
    [
        dcc.Location(id="url", refresh=False),
        dbc.Navbar(
            dbc.Container(
                [
                    html.Div(
                        children=[
                            html.Div(
                                html.Img(src=r"/assets/logos/ansys_solutions_logo_white.png", height="36px"),
                                style={
                                    "flex": "1",
                                    "text-align": "left",
                                },
                            ),
                            html.Div(
                                dmc.Group(
                                    [
                                        dmc.Text(
                                            "Project name:",
                                            id="project-name",
                                            size="sm",
                                            color="#FFFFFF",
                                            style={"font-size": "16px"},
                                        ),
                                        dmc.ActionIcon(
                                            DashIconify(icon="ph:code-fill", width=36),
                                            id="access-dev-guide",
                                            size=36,
                                            variant="transparent",
                                            style={"color": "#FFFFFF"},
                                        ),
                                        dbc.Popover(
                                            "Get access to the Solution's Developer Guide.",
                                            target="access-dev-guide",
                                            body=True,
                                            trigger="hover",
                                        ),
                                        html.Div(id="return-to-portal"),
                                    ],
                                    spacing=10,
                                ),
                                style={
                                    "display": "flex",
                                    "align-items": "flex-end",
                                },
                            ),
                        ],
                        style={
                            "display": "flex",
                            "justify-content": "space-between",
                            "width": "100%",
                        },
                    )
                ],
                style={"max-width": "inherit"},
                fluid=True,
            ),
            color="#000000",
            sticky="top",
            style={"height": "80px"},
        ),
        dbc.Row(
            [
                dbc.Col(
                    [
                        dmc.Space(h=10),
                        dbc.Row(
                            AwcDashTree(
                                id="navigation_tree",
                                multi=False,
                                height=250,
                                items=step_list,
                                selectedItemIds=["about_page"],
                            )
                        ),
                    ],
                    width=2,
                ),
                dbc.Col(html.Div(id="page-content", style={"padding-right": "0.7%"}), width=10),
            ],
        ),
    ]
)


@callback(
    Output("return-to-portal", "children"),
    Input("url", "pathname"),
)
def return_to_portal(pathname):
    """Display Solution Portal when back-to-portal button gets selected."""
    portal_ui_url = DashClient.get_portal_ui_url()
    children = (
        []
        if portal_ui_url is None
        else [
            html.A(
                [
                    dmc.ActionIcon(
                        DashIconify(icon="carbon:return", width=36),
                        id="back-to-projects-icon",
                        size=36,
                        variant="transparent",
                        style={"color": "#FFFFFF"},
                    ),
                    dbc.Popover(
                        "Back to Projects.",
                        target="back-to-projects-icon",
                        body=True,
                        trigger="hover",
                    ),
                ],
                href=portal_ui_url,
            )
        ]
    )
    return children


@callback(
    Output("project-name", "children"),
    Input("url", "pathname"),
)
def display_poject_name(pathname):
    """Display current project name."""
    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    return f"Project Name: {project.project_display_name}"


@callback(
    Output("access-dev-guide", "children"),
    Input("access-dev-guide", "n_clicks"),
    prevent_initial_call=True,
)
def access_dev_guide_documentation(n_clicks):
    """Open the Developer's Guide home page in the web browser."""
    webbrowser.open_new("https://dev-docs.solutions.ansys.com/index.html")
    raise PreventUpdate


@callback(
    Output("navigation_tree", "focusRequested"),
    Output("page-content", "children"),
    [
        Input("url", "pathname"),
        Input("navigation_tree", "treeItemClicked"),
    ],
    prevent_initial_call=True,
)
def display_page(pathname, value):
    """Display page content."""
    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    triggered_id = callback_context.triggered[0]["prop_id"].split(".")[0]
    focusRequested = ""
    if triggered_id == "url":
        return "", about_page.layout()
    if triggered_id == "navigation_tree":
        if value["id"] is None:
            page_layout = html.H1("Welcome!")
        elif value["id"] == "about_page":
            focusRequested = "about_page"
            page_layout = about_page.layout()
        elif value["id"] == "first_page":
            focusRequested = "first_page"
            page_layout = first_page.layout(project.steps.first_step)
        elif value["id"] == "second_page":
            focusRequested = "second_page"
            page_layout = second_page.layout(project.steps.second_step)
        return focusRequested, page_layout



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the first step."""


from ansys.saf.glow.client.dashclient import DashClient, callback
from ansys.solutions.dash_components.table import InputRow, OutputRow
from dash_extensions.enrich import Input, Output, State, dcc, html
from dash_iconify import DashIconify
import dash_mantine_components as dmc

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{cookiecutter.__solution_definition_name}}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.first_step import FirstStep


def layout(step: FirstStep):
    """Layout of the first step page."""
    return html.Div(
        [
            html.H1("First Step", className="display-3", style={"font-size": "48px", "fontWeight": "bold"}),
            html.P(
                "Compute the sum of two numbers.",
                className="lead",
                style={"font-size": "20px"},
            ),
            html.Hr(className="my-2"),
            html.Br(),
            InputRow(
                "number",
                "first-arg",
                "First Argument",
                row_default_value=step.first_arg,
                row_description="Enter a value.",
                label_width=2,
                value_width=4,
                unit_width=1,
                description_width=4,
                font_size="16px",
            ).get(),
            InputRow(
                "number",
                "second-arg",
                "Second Argument",
                row_default_value=step.second_arg,
                row_description="Enter a value.",
                label_width=2,
                value_width=4,
                unit_width=1,
                description_width=4,
                font_size="16px",
            ).get(),
            html.Br(),
            html.Div(
                dmc.Button(
                    "Calculate",
                    id="calculate",
                    leftIcon=DashIconify(icon="streamline:startup-solid"),
                    radius="xl",
                    disabled=False,
                    className="mantine-button",
                    style={"color": "#FFFFFF", "width": "100%", "background-color": "#000000", "font-size": "14px"},
                ),
                style={
                    "textAlign": "center",
                    "margin-left": "275px",
                    "width": "500px",
                },
            ),
            html.Br(),
            OutputRow(
                "number",
                "result",
                "Result",
                row_default_value=step.result,
                label_width=2,
                value_width=4,
                unit_width=1,
                description_width=4,
                class_name="button",
                font_size="16px",
            ).get(),
            dcc.Loading(
                type="circle",
                fullscreen=True,
                color="#ffb71b",
                style={
                    "background-color": "rgba(55, 58, 54, 0.1)",
                },
                children=html.Div(id="wait_completion"),
            ),
        ]
    )


@callback(
    Output("result", "value"),
    Output("wait_completion", "children"),
    Input("calculate", "n_clicks"),
    State("first-arg", "value"),
    State("second-arg", "value"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def calculate(n_clicks, first_arg, second_arg, pathname):
    """Trigger the computation."""
    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    step = project.steps.first_step
    step.first_arg = first_arg
    step.second_arg = second_arg
    step.calculate()
    return step.result, True



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""


from dash_extensions.enrich import html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.second_step import SecondStep


def layout(step: SecondStep):
    """Layout of the second step page."""
    return html.Div(
        [
            html.H1("Second Step", className="display-3", style={"font-size": "48px", "fontWeight": "bold"}),
            html.P(
                "This page is empty for now.",
                className="lead",
                style={"font-size": "20px"},
            ),
            html.Hr(className="my-2"),
            html.Br(),
        ]
    )



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the about page."""

import base64
import os
from pathlib import Path

import dash_bootstrap_components as dbc
from dash_extensions.enrich import html
import dash_mantine_components as dmc


def layout():
    """Layout of the about page."""
    solution_workflow_sketch_encoded = base64.b64encode(
        open(
            os.path.join(Path(__file__).parent.parent.absolute(), "assets", "images", "solution-workflow-sketch.png"),
            "rb",
        ).read()
    )

    return html.Div(
        dbc.Container(
            [
                html.H1("{{cookiecutter.solution_display_name}}", className="display-3", style={"font-size": "48px", "fontWeight": "bold"}),
                html.P(
                    "Add a short sentence to describe the goal of the solution.",
                    className="lead",
                    style={"font-size": "20px"},
                ),
                html.Hr(className="my-2"),
                dbc.Row(
                    [
                        dbc.Col(
                            [
                                dbc.Row(
                                    [
                                        dmc.Space(h=30),
                                        html.H4("Description", style={"font-size": "24px", "fontWeight": "bold"}),
                                        html.P(
                                            "Put here a short description of the solution.",
                                            style={"font-size": "16px", "textAlign": "justify"},
                                        ),
                                        dmc.Space(h=30),
                                        html.H4("Customer Goals", style={"font-size": "24px", "fontWeight": "bold"}),
                                        html.P(
                                            "What are the benefits for the customer?",
                                            style={"font-size": "16px", "textAlign": "justify"},
                                        ),
                                        dmc.Space(h=30),
                                        html.H4("Engineering Goals", style={"font-size": "24px", "fontWeight": "bold"}),
                                        html.P(
                                            "What is the engineering problem to solve? What are the Key Performance "
                                            "Indicators (KPIs) to monitor?",
                                            style={"font-size": "16px", "textAlign": "justify"},
                                        ),
                                    ]
                                )
                            ],
                            width=6,
                        ),
                        dbc.Col(
                            [
                                dbc.Row(
                                    [
                                        dbc.Card(
                                            [
                                                dbc.CardImg(
                                                    src="data:image/png;base64,{}".format(
                                                        solution_workflow_sketch_encoded.decode()
                                                    ),
                                                ),
                                                dbc.CardFooter("This is the introduction image."),
                                            ],
                                            style={"width": "50rem"},
                                        )
                                    ]
                                )
                            ],
                            width=6,
                        ),
                    ]
                ),
            ],
            fluid=True,
        ),
    )



from {{cookiecutter.__pkg_namespace}} import __version__


def test_pkg_version():
    import importlib.metadata as importlib_metadata

    # Read from the pyproject.toml
    # major, minor, patch
    read_version = importlib_metadata.version("{{ cookiecutter.__pkg_name }}")

    assert __version__ == read_version



"""{{ cookiecutter.product_name }}.{{ cookiecutter.library_name }} init file."""

import importlib.metadata as importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Sphinx documentation configuration file."""


from datetime import datetime
import os
from pathlib import Path

from ansys_sphinx_theme import ansys_favicon, get_version_match
import toml

BRANCH = "main"
ORGANIZATION_NAME = "ansys-internal"
DOC_PATH = "doc/source"


def remove_scheme_from_url(url: str) -> tuple:
    """Remove the scheme part of a URL."""
    items = url.split("://")
    if len(items) == 2:
        return items[0], items[1]
    else:
        raise Exception(f"Fail to return the URL without the scheme part for {url}.")


def get_repository_name_from_url(url: str) -> str:
    """Get repository name from the URL."""
    repository_name = remove_scheme_from_url(repository_url)[1].split("/")[-1]
    if ".git" in repository_name:
        repository_name = repository_name.split(".git")[0]
    return repository_name


# ---------- // Project Information // --------------------------------------------------------------------------------

configuration_file = Path(__file__).parent.parent.parent.absolute() / "pyproject.toml"
if configuration_file.exists():
    configuration = toml.load(configuration_file)
else:
    raise FileNotFoundError("No configuration file at project's root.")

project_name = configuration.get("tool", {}).get("poetry", {}).get("name", None)
package_version = configuration.get("tool", {}).get("poetry", {}).get("version", None)
documentation_url = configuration.get("tool", {}).get("poetry", {}).get("documentation", None)
repository_url = configuration.get("tool", {}).get("poetry", {}).get("repository", None)

if documentation_url:
    cname = remove_scheme_from_url(documentation_url)[1]
else:
    cname = None

if documentation_url:
    repository_name = remove_scheme_from_url(repository_url)
else:
    repository_name = None

copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS Inc."
switcher_version = get_version_match(package_version)


# ---------- // General configuration // ------------------------------------------------------------------------------

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
    "sphinx_code_tabs",
    "sphinx.ext.todo",
    "sphinxcontrib.autodoc_pydantic",
    "sphinx_design",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/dev", None),
}

# pydantic configuration
autodoc_pydantic_model_show_json = False
autodoc_pydantic_model_show_config = False
autodoc_pydantic_settings_show_json = False
autodoc_pydantic_model_show_validator_members = False
autodoc_pydantic_model_show_validator_summary = False

suppress_warnings = ["label.*"]

# numpydoc configuration
numpydoc_use_plots = True
numpydoc_show_class_members = False
numpydoc_xref_param_type = True
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}

# Favicon
html_favicon = ansys_favicon

# notfound.extension
notfound_template = "404.rst"
notfound_urls_prefix = "/../"

# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    "links.rst",
    "substitutions.rst",
]

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

autodoc_mock_imports = ["ansys.platform", "opentelemetry"]
numfig = True
numfig_secnum_depth = 1
numfig_format = {
    "figure": "Figure %s ",
    "table": "Table %s ",
    "code-block": "Code sample %s ",
}

# ---------- // Copy button customization // --------------------------------------------------------------------------

# exclude traditional Python prompts from the copied code
copybutton_prompt_text = r">>> ?|\.\.\. "
copybutton_prompt_is_regexp = True


# ---------- // Sphinx Gallery Options // -----------------------------------------------------------------------------


# ---------- // Options for HTML output // ----------------------------------------------------------------------------

html_short_title = html_title = "Ansys Solutions {{cookiecutter.solution_display_name}}"
html_theme = "ansys_sphinx_theme"
html_logo = str(Path(__file__).parent.absolute() / "_static" / "ansys-solutions-logo-black-background.png")
html_theme_options = {
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "collapse_navigation": True,
    "use_edit_page_button": False,
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": switcher_version,
    },
}

if str(os.getenv("DISABLE_GITHUB_URL_LINK")).lower() != "true":
    html_theme_options["github_url"] = f"https://github.com/{ORGANIZATION_NAME}/{repository_name}"

html_context = {
    "display_github": False,  # Integrate GitHub
    "github_user": ORGANIZATION_NAME,
    "github_repo": repository_name,
    "github_version": BRANCH,
    "doc_path": DOC_PATH,
}
html_show_sourcelink = False
html_compact_lists = False


# ---------- // Options for HTMLHelp output // ------------------------------------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "ansys-solutions-{{cookiecutter.__project_name_slug}}-documentation"


# ---------- // Options for LaTeX output // ---------------------------------------------------------------------------

latex_elements = {}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        f"Ansys-Solutions-{{cookiecutter.__project_name_slug}}-Documentation-{package_version}.tex",
        "Ansys Solutions {{cookiecutter.solution_display_name}} Documentation",
        author,
        "manual",
    ),
]


# ---------- // [Options for manual page output // --------------------------------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        master_doc,
        "Ansys Solutions {{cookiecutter.solution_display_name}}",
        "Ansys Solutions {{cookiecutter.solution_display_name}} Documentation",
        [author],
        1,
    )
]


# ---------- // Options for Texinfo output // -------------------------------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "Ansys Solutions {{cookiecutter.solution_display_name}}",
        "Ansys Solutions {{cookiecutter.solution_display_name}} Documentation",
        author,
        "Ansys Solutions {{cookiecutter.solution_display_name}}",
        "Engineering Software",
    ),
]

# Keep these while the repository is private
linkcheck_ignore = [
    "{{ cookiecutter.__repository_url }}/*",
    {%- if cookiecutter.__template_name == "pyansys-advanced" %}
    "{{ cookiecutter.__documentation_url }}/version/stable/*",
    {%- endif %}
    "https://pypi.org/project/{{cookiecutter.__pkg_name}}",
]

# If we are on a release, we have to ignore the "release" URLs, since it is not
# available until the release is published.
if switcher_version != "dev":
    linkcheck_ignore.append(f"https://github.com/ansys/{{ cookiecutter.__pkg_namespace }}/releases/tag/v{package_version}")
